/*
  Program code for the Pimp Your Chocolates project. 
  Charlieplexing 10 LEDs mounted into a chocolate box using ATtiny13 with some brightness control using pseudo-PWM
  
  The tilt switch motion sensor is connected to Pin 3 (PB4)
  See http://elabz.com/valentines-day-pov-display-using-attiny13-and-arduino-ide/
  for the complete circuit schematics
  
  Code by Elabz.com
  http://elabz.com/
  This example code is in the public domain. If you end up using it in a project, please drop me a message, I'd be happy to
  know it was of some use. I'll also be happy to feature your project on my site, so send some pictures, too.
  
  LED hookup can be gleaned from the bitmap[] array values. For example, first LED's value is B00000001 which means that
  to light the LED the D0 has to ho HIGH and D1 - LOW, so the LED's anode is facing D0. LED #2 is between the same legs but in reverse. 
  LED #3 is between D0 and D2, LED #4 is the same legs but in reverse and so on. 
  
//       ATMEL ATTINY13 / ARDUINO
//
//                 +-\/-+
// ADC0 (D 5) PB5 1|    |8 Vcc
// ADC3 (D 3) PB3 2|    |7 PB2 (D 2) ADC1
// ADC2 (D 4) PB4 3|    |6 PB1 (D 1) PWM1
//            GND 4|    |5 PB0 (D 0) PWM0
//                 +----+

  
  
 */
#include <avr/pgmspace.h>

#include <avr/sleep.h> // sleep code by insidegadgets.com
#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif
#ifndef sbi
#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#endif

// If your LEDs are laid out as on the schematics (LED6 is at the top)
//byte bitmap[] PROGMEM ={0B00000001,0B000000010,0B00000010,0B00000100,0B00000100,0B00000001};
//byte outModes[] PROGMEM ={0B00000011,0B00000011,0B00000110,0B00000110,0B00000101,0B00000101};

// If your LEDs are laid out in reversed order (LED6 is at the bottom)
byte bitmap[] PROGMEM ={0B00000100,0B00000001,0B00000010,0B00000100,0B00000001,0B000000010};
byte outModes[] PROGMEM ={0B00000101,0B00000101,0B00000110,0B00000110,0B00000011,0B00000011};


/* Valentine's day message
PROGMEM byte column[17]   = {

  33,    // --X----X // 0
  63,    // --XXXXXX // 1
  33,    // --X----X // 2
  0,     // -------- // 3
  24,    // ---XX--- // 4
  60,    // --XXXX-- // 5
  30,    // ---XXXX- // 6
  15,    // ----XXXX // 7
  30,    // ---XXXX- // 8
  60,    // --XXXX-- // 9
  24,    // ---XX--- // 10
  0,     // -------- // 11
  62,    // --XXXXX- // 12
  1,     // -------X // 13
  1,     // -------X // 14
  62,    // --XXXXX- // 15
  0      // -------- // 16

};
*/

/*St. Patrick's Day message */

PROGMEM byte column[30]   = {

  33,    // --X----X // 0
  63,    // --XXXXXX // 1
  33,    // --X----X // 2
  0,     // -------- // 3
  24,    // ---XX--- // 4
  60,    // --XXXX-- // 5
  30,    // ---XXXX- // 6
  15,    // ----XXXX // 7
  30,    // ---XXXX- // 8
  60,    // --XXXX-- // 9
  24,    // ---XX--- // 10
  0,     // -------- // 11
  63,    // --XXXXXX // 12
  41,    // --X-X--X // 13
  41,    // --X-X--X // 14
  22,    // ---X-XX- // 15
  0,     // -------- // 16
  63,    // --XXXXXX // 17
  41,    // --X-X--X // 18
  41,    // --X-X--X // 19
  0,     // -------- // 20
  63,    // --XXXXXX // 21
  41,    // --X-X--X // 22
  41,    // --X-X--X // 23
  0,     // -------- // 24
  63,    // --XXXXXX // 25
  44,    // --X-XX-- // 26 
  42,    // --X-X-X- // 27
  17,    // ---X---X // 28
  0      // -------- // 29 
};



byte y;
byte x;
byte ledDelay; // microseconds to show one LED
unsigned int showTime=10000;
int debounceDelay = 100; // microseconds
byte p=LOW; // last state
unsigned int swingTime;
unsigned int swingTimeAverage;
unsigned int swingDelay;
unsigned long swingTimeLast;
unsigned  long lastDebounceTime;

void setup() {
swingTimeLast =  millis();
}

void loop() {
unsigned long showNow;
unsigned  long periodNow;

byte currentColumn;
byte mask=1;

showNow = millis();
while(millis()-showNow < showTime)
{

int readNow=digitalRead(3); // 
if(readNow != p)
{
  lastDebounceTime = micros();
  
}

  if ((micros() - lastDebounceTime) > debounceDelay) {
    // whatever the reading is at, it's been there for longer
    // than the debounce delay, so take it as the actual current state:
    p=readNow;  
    
if(p==HIGH){ // show symbols only on one way
  y=0;
  
 // while(y < 17){ // Valentine's Day Message
     while(y < 30){ // St. Patrick's Day Message
     if(y==0) { delay(swingDelay); }  // delay showing first column because the beginning of the cycle is where the symbol does not look right (speed too low)
      x=0;
      currentColumn = pgm_read_word_near(&(column[y])); 
      mask=1;
      while(x<6){
         if(currentColumn & mask){
         PORTB = pgm_read_byte(&(bitmap[x])); // turn LED on
         DDRB = pgm_read_byte(&(outModes[x]));
         }else{
         DDRB = 0B00000000; 
         } 
        if(micros()-periodNow > ledDelay)
        {  
        x++;
        mask <<= 1;
        periodNow = micros();
        
        }       
      }
    y++;
  }

  swingTime = millis()-swingTimeLast;
  swingTimeAverage = (swingTimeAverage+swingTime)/2; 
  swingDelay=swingTimeAverage/4;
  ledDelay = swingDelay/256; // this only works because ledDelay is in microseconds
  swingTimeLast=millis();  

}
 
  lastDebounceTime = periodNow;
}

 sbi(GIMSK,PCIE); // Turn on Pin Change interrupt
 sbi(PCMSK,PCINT3); // Which pins are affected by the interrupt
 system_sleep();

  }

}

// From http://interface.khm.de/index.php/lab/experiments/sleep_watchdog_battery/
void system_sleep() {
  cbi(ADCSRA,ADEN); // Switch Analog to Digital converter OFF
  set_sleep_mode(SLEEP_MODE_PWR_DOWN); // Set sleep mode
  sleep_mode(); // System sleeps here
  sbi(ADCSRA,ADEN);  // Switch Analog to Digital converter ON
}

ISR(PCINT0_vect) {

}
